@@ -1,5 +1,7 @@
     /* buffer contains the memory required for the arguments structure */
-    buffer_size = sizeof(ffi_pl_argument) * self->ffi_cif.nargs * 2 + sizeof(ffi_pl_arguments);
+    buffer_size = sizeof(ffi_pl_argument) * self->ffi_cif.nargs +
+                  sizeof(void*) * self->ffi_cif.nargs +
+                  sizeof(ffi_pl_arguments);
 #ifdef HAVE_ALLOCA
     buffer = alloca(buffer_size);
 #else
@@ -8,6 +10,7 @@
     current_argv = arguments = (ffi_pl_arguments*) buffer;
 
     arguments->count = self->ffi_cif.nargs;
+    argument_pointers = (void**) &arguments->slot[arguments->count];
 
     /*
      * ARGUMENT IN
@@ -16,7 +19,7 @@
     for(i=0; i < self->ffi_cif.nargs; i++)
     {
       int platypus_type = self->argument_types[i]->platypus_type;
-      ((void**)&arguments->slot[arguments->count])[i] = &arguments->slot[i];
+      argument_pointers[i] = (void*) &arguments->slot[i];
 
       arg = i+(EXTRA_ARGS) < items ? ST(i+(EXTRA_ARGS)) : &PL_sv_undef;
       if(platypus_type == FFI_PL_NATIVE)
@@ -101,31 +104,31 @@
           switch(self->argument_types[i]->ffi_type->type)
           {
             case FFI_TYPE_UINT8:
-              Newx_or_alloca(ptr, uint8_t);
+              Newx_or_alloca(ptr, 1, uint8_t);
               *((uint8_t*)ptr) = SvOK(arg2) ? SvUV(arg2) : 0;
               break;
             case FFI_TYPE_SINT8:
-              Newx_or_alloca(ptr, int8_t);
+              Newx_or_alloca(ptr, 1, int8_t);
               *((int8_t*)ptr) = SvOK(arg2) ? SvIV(arg2) : 0;
               break;
             case FFI_TYPE_UINT16:
-              Newx_or_alloca(ptr, uint16_t);
+              Newx_or_alloca(ptr, 1, uint16_t);
               *((uint16_t*)ptr) = SvOK(arg2) ? SvUV(arg2) : 0;
               break;
             case FFI_TYPE_SINT16:
-              Newx_or_alloca(ptr, int16_t);
+              Newx_or_alloca(ptr, 1, int16_t);
               *((int16_t*)ptr) = SvOK(arg2) ? SvIV(arg2) : 0;
               break;
             case FFI_TYPE_UINT32:
-              Newx_or_alloca(ptr, uint32_t);
+              Newx_or_alloca(ptr, 1, uint32_t);
               *((uint32_t*)ptr) = SvOK(arg2) ? SvUV(arg2) : 0;
               break;
             case FFI_TYPE_SINT32:
-              Newx_or_alloca(ptr, int32_t);
+              Newx_or_alloca(ptr, 1, int32_t);
               *((int32_t*)ptr) = SvOK(arg2) ? SvIV(arg2) : 0;
               break;
             case FFI_TYPE_UINT64:
-              Newx_or_alloca(ptr, uint64_t);
+              Newx_or_alloca(ptr, 1, uint64_t);
 #ifdef HAVE_IV_IS_64
               *((uint64_t*)ptr) = SvOK(arg2) ? SvUV(arg2) : 0;
 #else
@@ -133,7 +136,7 @@
 #endif
               break;
             case FFI_TYPE_SINT64:
-              Newx_or_alloca(ptr, int64_t);
+              Newx_or_alloca(ptr, 1, int64_t);
 #ifdef HAVE_IV_IS_64
               *((int64_t*)ptr) = SvOK(arg2) ? SvIV(arg2) : 0;
 #else
@@ -141,15 +144,15 @@
 #endif
               break;
             case FFI_TYPE_FLOAT:
-              Newx_or_alloca(ptr, float);
+              Newx_or_alloca(ptr, 1, float);
               *((float*)ptr) = SvOK(arg2) ? SvNV(arg2) : 0.0;
               break;
             case FFI_TYPE_DOUBLE:
-              Newx_or_alloca(ptr, double);
+              Newx_or_alloca(ptr, 1, double);
               *((double*)ptr) = SvOK(arg2) ? SvNV(arg2) : 0.0;
               break;
             case FFI_TYPE_POINTER:
-              Newx_or_alloca(ptr, void*);
+              Newx_or_alloca(ptr, 1, void*);
               {
                 SV *tmp = SvRV(arg);
                 *((void**)ptr) = SvOK(tmp) ? INT2PTR(void *, SvIV(tmp)) : NULL;
@@ -409,7 +412,52 @@
         for(n=0; n < self->argument_types[i]->extra[0].custom_perl.argument_count; n++)
         {
           i++;
-          ((void**)&arguments->slot[arguments->count])[i] = &arguments->slot[i];
+          argument_pointers[i] = &arguments->slot[i];
+        }
+      }
+      else if(platypus_type == FFI_PL_EXOTIC_FLOAT)
+      {
+        switch(self->argument_types[i]->ffi_type->type)
+        {
+#ifdef SIZEOF_LONG_DOUBLE
+          case FFI_TYPE_LONGDOUBLE:
+            {
+              long double *ptr;
+              Newx_or_alloca(ptr, 1, long double);
+              argument_pointers[i] = ptr;
+              ffi_pl_perl_long_double(arg, ptr);
+            }
+            break;
+#endif
+#ifdef FFI_PL_PROBE_COMPLEX
+          case FFI_TYPE_COMPLEX:
+            switch(self->argument_types[i]->ffi_type->size)
+            {
+              case  8:
+                {
+                  float *ptr;
+                  Newx_or_alloca(ptr, 2, float complex);
+                  argument_pointers[i] = ptr;
+                  ffi_pl_perl_complex_float(arg, ptr);
+                }
+                break;
+              case 16:
+                {
+                  double *ptr;
+                  Newx_or_alloca(ptr, 2, double);
+                  argument_pointers[i] = ptr;
+                  ffi_pl_perl_complex_double(arg, ptr);
+                }
+                break;
+              default :
+                warn("argument type not supported (%d)", i);
+                break;
+            }
+            break;
+#endif
+          default:
+            warn("argument type not supported (%d)", i);
+            break;
         }
       }
       else
@@ -426,16 +474,43 @@
     fprintf(stderr, "# ===[%p]===\n", self->address);
     for(i=0; i < self->ffi_cif.nargs; i++)
     {
-      fprintf(stderr, "# [%d] <%d:%d> %p %p %016llx \n",
+      fprintf(stderr, "# [%d] <%d:%d> %p %p",
         i,
         self->argument_types[i]->ffi_type->type,
         self->argument_types[i]->platypus_type,
-        ((void**)&arguments->slot[arguments->count])[i],
-        &arguments->slot[i],
-        ffi_pl_arguments_get_uint64(arguments, i)
-        /* ffi_pl_arguments_get_float(arguments, i), *
-         * ffi_pl_arguments_get_double(arguments, i) */
+        argument_pointers[i],
+        &arguments->slot[i]
       );
+      if(self->argument_types[i]->platypus_type  == FFI_PL_EXOTIC_FLOAT)
+      {
+        switch(self->argument_types[i]->ffi_type->type)
+        {
+          case FFI_TYPE_LONGDOUBLE:
+            fprintf(stderr, " %Lg", *((long double*)argument_pointers[i]));
+            break;
+          case FFI_TYPE_COMPLEX:
+            switch(self->argument_types[i]->ffi_type->size)
+            {
+              case 8:
+                fprintf(stderr, " %g + %g * i",
+                  crealf(*((float complex*)argument_pointers[i])),
+                  cimagf(*((float complex*)argument_pointers[i]))
+                );
+                break;
+              case 16:
+                fprintf(stderr, " %g + %g * i",
+                  creal(*((double complex*)argument_pointers[i])),
+                  cimag(*((double complex*)argument_pointers[i]))
+                );
+                break;
+            }
+        }
+      }
+      else
+      {
+        fprintf(stderr, "%016llx", ffi_pl_arguments_get_uint64(arguments, i));
+      }
+      fprintf(stderr, "\n");
     }
     fprintf(stderr, "# === ===\n");
     fflush(stderr);
@@ -461,7 +536,10 @@
 
     for(i=self->ffi_cif.nargs-1; i >= 0; i--)
     {
-      if(self->argument_types[i]->platypus_type == FFI_PL_POINTER)
+      platypus_type platypus_type;
+      platypus_type = self->argument_types[i]->platypus_type;
+    
+      if(platypus_type == FFI_PL_POINTER)
       {
         void *ptr = ffi_pl_arguments_get_pointer(arguments, i);
         if(ptr != NULL)
@@ -522,8 +600,7 @@
         Safefree(ptr);
 #endif
       }
-
-      if(self->argument_types[i]->platypus_type == FFI_PL_ARRAY)
+      else if(platypus_type == FFI_PL_ARRAY)
       {
         void *ptr = ffi_pl_arguments_get_pointer(arguments, i);
         int count = self->argument_types[i]->extra[0].array.element_count;
@@ -620,7 +697,7 @@
         Safefree(ptr);
 #endif
       }
-      else if(self->argument_types[i]->platypus_type == FFI_PL_CLOSURE)
+      else if(platypus_type == FFI_PL_CLOSURE)
       {
         arg = i+(EXTRA_ARGS) < items ? ST(i+(EXTRA_ARGS)) : &PL_sv_undef;
         if(SvROK(arg))
@@ -628,7 +705,7 @@
           SvREFCNT_dec(arg);
         }
       }
-      else if(self->argument_types[i]->platypus_type == FFI_PL_CUSTOM_PERL)
+      else if(platypus_type == FFI_PL_CUSTOM_PERL)
       {
         /* FIXME: need to fill out argument_types for skipping */
         i -= self->argument_types[i]->extra[0].custom_perl.argument_count;
@@ -641,6 +718,13 @@
           }
         }
       }
+#ifndef HAVE_ALLOCA
+      else if(platypus_type == FFI_PL_EXOTIC_FLOAT)
+      {
+        void *ptr = argument_pointers[i];
+        Safefree(ptr);
+      }
+#endif
     }
 #ifndef HAVE_ALLOCA
     if(self->return_type->platypus_type != FFI_PL_CUSTOM_PERL)
@@ -665,16 +749,32 @@
         switch(self->return_type->ffi_type->type)
         {
           case FFI_TYPE_UINT8:
+#ifdef FFI_PL_PROBE_BIGENDIAN
+            XSRETURN_UV(result.uint8_array[3]);
+#else
             XSRETURN_UV(result.uint8);
+#endif
             break;
           case FFI_TYPE_SINT8:
+#ifdef FFI_PL_PROBE_BIGENDIAN
+            XSRETURN_IV(result.sint8_array[3]);
+#else
             XSRETURN_IV(result.sint8);
+#endif
             break;
           case FFI_TYPE_UINT16:
+#ifdef FFI_PL_PROBE_BIGENDIAN
+            XSRETURN_UV(result.uint16_array[1]);
+#else
             XSRETURN_UV(result.uint16);
+#endif
             break;
           case FFI_TYPE_SINT16:
+#ifdef FFI_PL_PROBE_BIGENDIAN
+            XSRETURN_IV(result.sint16_array[1]);
+#else
             XSRETURN_IV(result.sint16);
+#endif
             break;
           case FFI_TYPE_UINT32:
             XSRETURN_UV(result.uint32);
@@ -945,16 +1045,32 @@
       switch(self->return_type->ffi_type->type)
       {
         case FFI_TYPE_UINT8:
+#ifdef FFI_PL_PROBE_BIGENDIAN
+          ret_in = newSVuv(result.uint8_array[3]);
+#else
           ret_in = newSVuv(result.uint8);
+#endif
           break;
         case FFI_TYPE_SINT8:
+#ifdef FFI_PL_PROBE_BIGENDIAN
+          ret_in = newSViv(result.sint8_array[3]);
+#else
           ret_in = newSViv(result.sint8);
+#endif
           break;
         case FFI_TYPE_UINT16:
+#ifdef FFI_PL_PROBE_BIGENDIAN
+          ret_in = newSVuv(result.uint16_array[1]);
+#else
           ret_in = newSVuv(result.uint16);
+#endif
           break;
         case FFI_TYPE_SINT16:
+#ifdef FFI_PL_PROBE_BIGENDIAN
+          ret_in = newSViv(result.sint16_array[1]);
+#else
           ret_in = newSViv(result.sint16);
+#endif
           break;
         case FFI_TYPE_UINT32:
           ret_in = newSVuv(result.uint32);
@@ -1024,6 +1140,32 @@
       }
 
     }
+    else if(self->return_type->platypus_type == FFI_PL_EXOTIC_FLOAT)
+    {
+      switch(self->return_type->ffi_type->type)
+      {
+#ifdef SIZEOF_LONG_DOUBLE
+        case FFI_TYPE_LONGDOUBLE:
+        {
+          if(have_math_longdouble)
+          {
+            SV *sv;
+            long double *ptr;
+            Newx(ptr, 1, long double);
+            *ptr = result.longdouble;
+            sv = sv_newmortal();
+            sv_setref_pv(sv, "Math::LongDouble", (void*)ptr);
+            ST(0) = sv;
+            XSRETURN(1);
+          }
+          else
+          {
+            XSRETURN_NV((double) result.longdouble);
+          }
+        }
+#endif
+      }
+    }
 
     warn("return type not supported");
     XSRETURN_EMPTY;
