@@ -6,7 +6,7 @@ use 5.008001;
 use Carp qw( croak );
 
 # ABSTRACT: Write Perl bindings to non-Perl libraries with FFI. No XS required.
-our $VERSION = '0.22'; # VERSION
+our $VERSION = '0.23'; # VERSION
 
 # Platypus Man,
 # Platypus Man,
@@ -78,9 +78,17 @@ sub _type_map
     {
       croak "$class does not provide a native_type_map method";
     }
-    my %type_map = %{ $class->native_type_map };
+    my %type_map;
+    foreach my $key (keys %{ $class->native_type_map  })
+    {
+      my $value = $class->native_type_map->{$key};
+      next unless _have_type($value);
+      $type_map{$key} = $value;
+    }
     # include the standard libffi types
-    $type_map{$_} = $_ for qw( void sint8 uint8 sint16 uint16 sint32 uint32 sint64 uint64 float double string opaque );
+    $type_map{$_} = $_ for grep { _have_type($_) } 
+      qw( void sint8 uint8 sint16 uint16 sint32 uint32 sint64 uint64 float double string opaque
+          longdouble complex_float complex_double );
     $type_map{pointer} = 'opaque';
     $self->{type_map} = \%type_map;
   }
@@ -473,9 +481,17 @@ sub DESTROY
   delete $self->{handles};
 }
 
+sub _have_pm
+{
+  my($class) = @_;
+  my $ok = eval qq{ use $class; 1 };
+  $ok = $ok ? $ok : 0;
+  $ok;
+}
+
 package FFI::Platypus::Function;
 
-our $VERSION = '0.22'; # VERSION
+our $VERSION = '0.23'; # VERSION
 
 use overload '&{}' => sub {
   my $ffi = shift;
@@ -487,7 +503,7 @@ package FFI::Platypus::Closure;
 use Scalar::Util qw( refaddr);
 use Carp qw( croak );
 
-our $VERSION = '0.22'; # VERSION
+our $VERSION = '0.23'; # VERSION
 
 our %cbdata;
 
@@ -514,13 +530,13 @@ sub DESTROY
 
 package FFI::Platypus::ClosureData;
 
-our $VERSION = '0.22'; # VERSION
+our $VERSION = '0.23'; # VERSION
 
 package FFI::Platypus::Type;
 
 use Carp qw( croak );
 
-our $VERSION = '0.22'; # VERSION
+our $VERSION = '0.23'; # VERSION
 
 sub new
 {
@@ -614,7 +630,7 @@ FFI::Platypus - Write Perl bindings to non-Perl libraries with FFI. No XS requir
 
 =head1 VERSION
 
-version 0.22
+version 0.23
 
 =head1 SYNOPSIS
 
@@ -1534,7 +1550,11 @@ implemented using FFI called L<ZMQ::FFI>.
  # https://github.com/libarchive/libarchive/wiki/Examples#List_contents_of_Archive_stored_in_File
  
  my $archive_filename = shift @ARGV;
- die "usage: $0 archive.tar" unless defined $archive_filename;
+ unless(defined $archive_filename)
+ {
+   print "usage: $0 archive.tar\n";
+   exit;
+ }
  
  my $archive = ArchiveRead->new;
  $archive->support_filter_all;
